@@ -16,7 +16,7 @@
 
 #include <KAboutData>
 #include <KDBusService>
-#include <KLocalizedContext>
+#include <KLocalizedQmlContext>
 #include <KLocalizedString>
 #include <KWindowSystem>
 
@@ -28,7 +28,6 @@
 
 #include <iostream>
 
-#include "controller.h"
 #include "filemenu.h"
 #include "filesystemtracker.h"
 #include "imagestorage.h"
@@ -65,7 +64,7 @@ int main(int argc, char **argv)
                          QStringLiteral(KOKO_VERSION_STRING),
                          xi18nc("@title", "Photos is an image viewer for your image collection."),
                          KAboutLicense::LGPL,
-                         xi18nc("@info:credit", "(c) 2013-2020 KDE Contributors"),
+                         xi18nc("@info:credit", "(c) 2013-%1 KDE Contributors", QString::number(QDate::currentDate().year())),
                          QString(),
                          QStringLiteral("https://apps.kde.org/koko"));
 
@@ -105,11 +104,11 @@ int main(int argc, char **argv)
 
     QThread trackerThread;
 
-    QStringList locations = QStandardPaths::standardLocations(QStandardPaths::PicturesLocation);
+    const QStringList locations = QStandardPaths::standardLocations(QStandardPaths::PicturesLocation);
     Q_ASSERT(locations.size() >= 1);
     qDebug() << locations;
 
-    QUrl currentDirPath = QUrl::fromLocalFile(QDir::currentPath().append('/'));
+    const QUrl currentDirPath = QUrl::fromLocalFile(QDir::currentPath().append('/'));
 
     QStringList directoryUrls;
     for (const auto &path : parser.positionalArguments()) {
@@ -134,23 +133,8 @@ int main(int argc, char **argv)
     trackerThread.start();
     tracker.setSubFolder(tracker.folder());
 
-    KokoConfig config;
-    // General group
-    QObject::connect(&config, &KokoConfig::IconSizeChanged, &config, &KokoConfig::save);
-    QObject::connect(&config, &KokoConfig::ImageViewPreviewChanged, &config, &KokoConfig::save);
-    QObject::connect(&config, &KokoConfig::SavedFoldersChanged, &config, &KokoConfig::save);
-    // Slideshow group
-    QObject::connect(&config, &KokoConfig::NextImageIntervalChanged, &config, &KokoConfig::save);
-    QObject::connect(&config, &KokoConfig::LoopImagesChanged, &config, &KokoConfig::save);
-    QObject::connect(&config, &KokoConfig::RandomizeImagesChanged, &config, &KokoConfig::save);
-    // KokoConfig WindowState group
-    QObject::connect(&config, &KokoConfig::WidthChanged, &config, &KokoConfig::save);
-    QObject::connect(&config, &KokoConfig::HeightChanged, &config, &KokoConfig::save);
-    QObject::connect(&config, &KokoConfig::VisibilityChanged, &config, &KokoConfig::save);
-    QObject::connect(&config, &KokoConfig::ControlsVisibleChanged, &config, &KokoConfig::save);
-
     QQmlApplicationEngine engine;
-    engine.rootContext()->setContextObject(new KLocalizedContext(&engine));
+    KLocalization::setupLocalizedContext(&engine);
 
     OpenFileModel openFileModel(directoryUrls);
     QObject::connect(&service,
@@ -183,11 +167,7 @@ int main(int argc, char **argv)
     qmlRegisterType<VectorImage>("org.kde.koko.image", 1, 0, "VectorImage");
     qmlRegisterType<FileMenu>("org.kde.koko.private", 1, 0, "FileMenu");
 
-    Controller controller;
-    qmlRegisterSingletonInstance("org.kde.koko.private", 1, 0, "Controller", &controller);
-
     engine.rootContext()->setContextProperty("kokoProcessor", &processor);
-    engine.rootContext()->setContextProperty("kokoConfig", &config);
 
     // we want different main files on desktop or mobile
     // very small difference as they as they are subclasses of the same thing
