@@ -6,11 +6,16 @@
 
 #ifndef ITEMTYPES_H
 #define ITEMTYPES_H
+#include "exiv2extractor.h"
 #include <QObject>
+#include <qqmlintegration.h>
 
 class Types : public QObject
 {
     Q_OBJECT
+    QML_ELEMENT
+    QML_UNCREATABLE("ENUM")
+
     Q_ENUMS(ItemTypes)
     Q_ENUMS(TimeGroup)
     Q_ENUMS(LocationGroup)
@@ -28,4 +33,11 @@ public:
     enum QueryType { LocationQuery = 10, TimeQuery };
 };
 
+class Exiv2ExtractorForeign
+{
+    Q_GADGET
+    QML_NAMED_ELEMENT(Exiv2Extractor)
+    QML_FOREIGN(Exiv2Extractor)
+};
+
 #endif
